/**
 * Hjälpmedel för att testa WCAG 1.4.4.
 *
 * Ett formulär där användaren kan fylla i den beräknade textstorleken vid 100 %
 * zoom och vid en högre förstoringsgrad, och få hjälp med att räkna ut om
 * kriterium 1.4.4 är uppfyllt.
 */
((doc) => {
    "use strict";


    /**
     * Formulärvy för kontroll av textförstoring.
     *
     * Vyn klonar ett formulär från ett mallelement i HTML-koden och hanterar
     * händelser i formuläret.
     */
    class FormView {

        /**
         * Konstruera en vyinstans.
         *
         * @param {HTMLTemplateElement} template TEMPLATE-element.
         * @param {TextZoom}            textZoom Beräkningsfunktion.
         */
        constructor (template, textZoom) {
            this._form   = this._cloneTemplate(template);
            this._output = this._form.querySelector("output");
            this._zoom   = textZoom;

            this._form.addEventListener("submit", (event) => {
                event.stopPropagation();
                event.preventDefault();
                this._form.reportValidity();
            });

            this._form.addEventListener("change", () => this._update());
        }

        /**
         * Klona ett mallelement.
         *
         * @private
         * @param {HTMLTemplateElement} template TEMPLATE-element.
         */
        _cloneTemplate (template) {
            template.parentNode.insertBefore(
                template.content.cloneNode(true),
                template
            );

            const form = template.previousElementSibling;

            form.querySelectorAll("[data-id]").forEach((input) => {
                input.id = input.dataset.id;
            });

            form.querySelectorAll("[data-for]").forEach((label) => {
                label.htmlFor = label.dataset["for"];
            });

            return form;
        }

        /**
         * Uppdatera visningsfältet.
         *
         * @private
         */
        _update () {
            if (!this._form.checkValidity()) {
                return;
            }

            const [
                orig,
                resized,
                zoom
            ] = Array.from(this._form.querySelectorAll("input"))
            .map((input) => input.valueAsNumber);

            const effective = this._zoom.effectiveSize(resized, zoom)
                              * 100 / orig;
            const compliant = this._zoom.compliant(orig, resized, zoom)
                              ? "godkänt" : "underkänt";

            this._output.value = `${Math.round(effective)} % (${compliant})`;
        }

    }


    /**
     * Beräknar effektiv förstoring av text vid följsam formgivning.
     *
     * Exempel: Om den angivna textstorleken vid 100 % zoom är 16px och vid
     * 400 % är 8px är 1.4.4 uppfyllt. Detta då den effektiva textstorleken vid
     * 400 % är 8·4=32px, vilket jämfört med 16px är 200 %.
     */
    class TextZoom {

        /**
         * Kontrollera om WCAG 1.4.4 är uppfyllt.
         *
         * Den effektiva storleken ska motsvara förstoringsgraden, eller
         * åtminstone dubbla storleken vid förstoringsgrader över 200 %.
         *
         * @param {number} original Beräknad textstorlek vid 100 % zoom.
         * @param {number} resized  Beräknad textstorlek vid angiven zoom.
         * @param {number} zoom     Förstoringsgrad (%).
         * @reutnr {boolean}
         */
        compliant (original, resized, zoom) {
            return this.effectiveSize(resized, zoom) / original
                   >= Math.min(zoom / 100, 2);
        }

        /**
         * Beräkna den effektiva textstorleken vid zoom.
         *
         * @param {number} computed Beräknad textstorlek.
         * @param {number} zoom     Förstoringsgrad (%).
         * @return {number}
         */
        effectiveSize (computed, zoom) {
            return computed * zoom / 100;
        }

    }


    /*
     * Initiering.
     */

    // Kontrollera kompatibilitet.
    if ("undefined" === typeof doc.querySelectorAll) {
        return;
    }

    // Skapa formulär.
    new FormView(doc.getElementById("text-zoom"), new TextZoom());
})(document);
